/* hdup.c, a backup program
 *
 * Author: Miek Gieben
 *
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

unsigned int quiet = 0;
char *scheme[] =  {"daily","weekly","monthly","remote","restore"};

void backup_remote(phost_t host[], int which, char *date) {

        /* get archive out of tmp/hdup */
        /* move to local global place */

        char *archive;

        /* set a lock */
        if (hdup_lock(host[which], LOCK) != 0 ) {
                /* could not get LOCK */
                hdup_cleanup(1, host[which]);
        }

        archive = (char*)malloc ( strlen(host[which]->name) + 11 +
                        19 +    /* dots and consts */
                        8);     /* scheme */

        /* get everything thats adheres to this format to the
         * backupdir 
         */
        sprintf(archive, "%s/%s.*.*.tar.*", host[which]->remote_path,host[which]->name);
        
        /* check if the is an archive */
        if (hdup_globfilecheck(archive,NULL) == 0 ) {
                rmdir(host[which]->dirname_date);
                rmdir(host[which]->basename);
                hdup_log("No archive found");
                hdup_overview("Status", "no archive found");
                hdup_cleanup(-1, host[which]);
        }

        if (hdup_mv(archive,host[which]->dirname_date ) != 0 ) {
                /* cleanup our mess */
                rmdir(host[which]->dirname_date);
                rmdir(host[which]->basename);
                hdup_overview("Status","could not perform the move");
                hdup_cleanup(-1, host[which]);
        }

        free(archive);
        rmdir(host[which]->dirname_etc);
        /* done !?!?! */
}

void backup_local(phost_t host[], int which, struct argument_t arg) {

        /* fill in the rest */
        hdup_tarlist( host[which]);
        /* the exclude list */
        hdup_excludelist(host[which]);

        hdup_archive(host[which], arg.scheme);

        hdup_overview("Archive",(char*)hdup_lastname(host[which]->archivename));
        hdup_overview("User",host[which]->user);

        if ( host[which]->alg != NULL ) {
                char *what; size_t l;
                l = strlen(host[which]->alg) + strlen(host[which]->keypath) + 15;
                what = (char*)malloc(l);
                sprintf(what,"yes (%s, %s)", host[which]->alg, host[which]->keypath);
                hdup_overview("Encryption", what);
                free(what);
        } else {
                hdup_overview("Encryption", "no");
        }

        hdup_inclist(host[which], arg.scheme);

        /* set a lock */
        if (hdup_lock(host[which], LOCK) != 0 ) {
                /* could not get LOCK */
                hdup_log("Could not get LOCK");
                hdup_cleanup(1, host[which]);
        }

        if ( hdup_tar(host[which]) != 0 ) {
                /* backup failed */
                hdup_log("Could not perform the backup");
                hdup_cleanup(1, host[which]);
        }
        if ( host[which]->alg != NULL ) 
                hdup_crypt(host[which]);

        if ( host[which]->remote_host != NULL ) {
                /* scp archivename user@host:/tmp/<archivename> */
                /* can also supply identity option -I */

                hdup_overview(NULL, NULL);
                hdup_overview("Remote host",host[which]->remote_host);
                if (host[which]->scp_user != NULL ) 
                        hdup_overview("Remote user",host[which]->scp_user);
                else
                        hdup_overview("Remote user",host[which]->user);

		/* Need this here, later archive is gone */
		hdup_overview("Archive size", (char*) (hdup_size (host[which]->archivename)));

                if ( hdup_scp(host[which]) != 0 ) {
                        /* hmmmm, something wrong */
                        if ( host[which]->remove == 1 )
                                hdup_log("Remote copy failed, not removing local copy");
                } else {
                        if ( host[which]->remove == 1 ) {
                                unlink(host[which]->archivename);
                                rmdir(host[which]->dirname_date); /* get ride of extra dir */
                        }
                }
        }
        /* done !?!? */
}

int main (int argc, char *argv[]) {

        phost_t host[MAXHOST + 1];
        struct option_t opt; struct argument_t arg;
        int c = 0; int which;

        /* reset options */
        opt.help = opt.remove = opt.verbose = opt.oldtar = 0;
        opt.identity    = NULL; opt.keypath     = NULL;
        opt.configpath  = NULL; opt.alg         = NULL;
        opt.tar         = NULL; opt.user        = NULL;
        opt.scp_user    = NULL;

        arg.scheme      = -1;   arg.host        = NULL;
        arg.date        = NULL; arg.extractdir  = NULL; 

        /* in host[0] I put the global options, these can be overriden
         * by the config file
         */
        while ( (c = getopt(argc, argv, "C:u:s:c:i:a:k:hroq")) != -1) {
                 switch (c) { 
                         case 'u':
                                opt.user = optarg;
                                break;
                         case 's':
                                opt.scp_user = optarg;
                                break;
                         case 'a':
                                opt.alg = optarg;
                                break;
                         case 'i':
                                opt.identity = optarg;
                                break;
                         case 'k':
                                opt.keypath = optarg;
                                break;
                         case 'c':
                                opt.configpath = optarg;
                                break;
                         case 'h':
                                opt.help = 1;
                                break;
                         case 'r':
                                opt.remove = 1;
                                break;
                         case 'C':
                                opt.tar = optarg;
                                break;
                         case 'o':
                                opt.oldtar = 1;
                                break;
                         case 'q':
                                quiet = 1;
                                break;
                         case '?':
                         default:
                                usage();
                                hdup_cleanup(-1,NULL);
                 }
        }

        argc -= optind;
        argv += optind;

        if ( opt.help == 1 ) { usage(); hdup_cleanup(-1,NULL); }

        /* first the config file to set things up */
        /* -c option */
        if ( opt.configpath != NULL ) {
                /* got an config option */
                if ( hdup_checkfile(opt.configpath) == 1  )
                        /* there is a config file */
                        if ( config(opt.configpath, host) != 0 ) {
                                /* something went wrong when reading
                                 * the configfile
                                 */
                                hdup_cleanup(-1,NULL);
                        }
        } else {
                hdup_log("No configuration file found, use -c option");
                hdup_cleanup(-1,NULL);
        }
        /* config checks */
        if ( findhost("global",host) == -1 ) {
                hdup_log("No [global] section in configuration file");
                hdup_cleanup(-1,NULL);
        }
        /* -r option */
        host[0]->remove = opt.remove;
        /* -o option */
        host[0]->oldtar = opt.oldtar;

        /* check -C */
        if ( opt.tar != NULL ) {
                host[0]->tar_opt = (char*)malloc(5);
                if (  strcmp (opt.tar, "gzip") == 0 ) {
                        /* found */
                        host[0]->tar_opt    = TAR_GZIP;
                } else {
                        if ( strcmp(opt.tar, "none") == 0 ) {
                                host[0]->tar_opt  = TAR_NONE;
                        } else {
                                if ( opt.oldtar == 1 )  {
                                        host[0]->tar_opt    = TAR_BZIPO;
                                } else {
                                        host[0]->tar_opt    = TAR_BZIP;
                                }
                        }
                }
        } else {
               host[0]->tar_opt = TAR_BZIP; /* default */
        }
        /* -u: check user */
        if ( opt.user == NULL ) {
                hdup_log("No user given (-u option) defaulting to \'operator\'");
                host[0]->user =  "operator";
        } else {
                host[0]->user = opt.user;
        }
        /* -s option */
        if ( opt.scp_user == NULL )
                host[0]->scp_user =  host[0]->user;
        else
                host[0]->scp_user = opt.scp_user;

        /* both must be set */
        if ( ( opt.alg == NULL && opt.keypath != NULL ) || ( opt.alg != NULL && opt.keypath == NULL ) ) {
                hdup_log("Both algorithm (-a) and keypath (-k) must be set");
                hdup_cleanup(-1,NULL);
        }
        if ( opt.alg != NULL ) {
                /* do we have mcrypt? */
                if (  hdup_checkfile (MCRYPT) == 0 ) {
                        hdup_log("Mcrypt not found, needed for encryption");
                        hdup_cleanup(-1,NULL);
                } else 
                        host[0]->alg = opt.alg;
                /* is key availible */
                if ( hdup_checkfile (opt.keypath) == 0 ) {
                        hdup_log("Encryption key not found");
                        hdup_cleanup(-1,NULL);
                } else
                        host[0]->keypath = opt.keypath; 
        }
        
        /* need at least two arguments: scheme and a hostname */
        if (argc < 2 ) {
                hdup_log("Not enough arguments");
                hdup_cleanup(-1,NULL);
        }

        /* arg: scheme */
        if ( strncmp (argv[0], scheme[DAILY], 6) == 0 )
                arg.scheme = DAILY;
        if ( strncmp (argv[0], scheme[WEEKLY], 7) == 0 ) 
                arg.scheme = WEEKLY;
        if ( strncmp (argv[0], scheme[MONTHLY], 8) == 0 ) 
                arg.scheme = MONTHLY;
        if ( strncmp (argv[0], scheme[REMOTE], 7) == 0 ) 
                arg.scheme = REMOTE;
        if ( strncmp (argv[0], scheme[RESTORE], 8) == 0 ) 
                arg.scheme = RESTORE;

        /* still -1? */
        if ( arg.scheme == -1 ) {
                hdup_log("Scheme must be: daily, weekly, monthly, remote or restore");
                hdup_cleanup(-1,NULL);
        }

        /* LOCAL */
        if ( arg.scheme < REMOTE ) {
                /* second arg: host to backup */
                arg.host = argv[1];

                if ( argv[2] != NULL ) {
                        /* must start with @ */
                        if ( ( strchr(argv[2], '@')) == argv[2] ) {
                                /* do not copy the @ */
                                host[0]->remote_host = ++argv[2];
                                if ( opt.identity != NULL && hdup_checkfile(opt.identity) == 0 ) {
                                        hdup_log("Identity file not found");
                                        hdup_cleanup(-1,NULL);
                                } else 
                                        host[0]->identity = opt.identity;
                        } else {
                                        usage();
                                        hdup_cleanup(-1,NULL);
                        }
                }
        }

        /* REMOTE */
        if ( arg.scheme == REMOTE ) 
                /* REMOTE */
                arg.host = argv[1];

        /* RESTORE */
        if ( arg.scheme == RESTORE ) {
                /* restore a backup */
                if (argv[1] != NULL) 
                        arg.host = argv[1];
                else
                        hdup_log("You must supply a hostname"),hdup_cleanup(-1,NULL);

                if (argv[2] != NULL) 
                        arg.date = argv[2];
                else
                        hdup_log("You must supply a date (DD-MM-YYYY)"),hdup_cleanup(-1,NULL);
                if (argv[3] != NULL) {
                        arg.extractdir  = argv[3];
                        if ( hdup_globfilecheck(arg.extractdir,NULL) == 0 ) {
                                hdup_log("Extract dir doesn't exist");
                                hdup_cleanup(-1,NULL);
                        }
                        /* check for the / dir? */
                } else {
                        hdup_log("You must supply a dir to extract to");
                        hdup_cleanup(-1,NULL);
                }
        }
        which = findhost(arg.host, host);
        if ( which == -1 ) {
                hdup_log("Host not defined in configuration file");
                hdup_cleanup(-1,NULL);
        }

        /* ok ready, copy some vars over
         * conditional, can be overridden in config
         */
        if ( host[which]->tar_opt == (char*) NULL )
                host[which]->tar_opt = host[0]->tar_opt;
        if ( host[which]->user == (char*) NULL )
                host[which]->user = host[0]->user;
        if ( host[which]->scp_user == (char*) NULL )
                host[which]->scp_user = host[0]->scp_user;
        if ( host[which]->remote_host == (char*) NULL )
                host[which]->remote_host = host[0]->remote_host;
        if ( host[which]->identity == (char*) NULL )
                host[which]->identity = host[0]->identity;
        if ( host[which]->keypath == (char*) NULL )
                host[which]->keypath = host[0]->keypath;
        if ( host[which]->alg == (char*) NULL )
                host[which]->alg = host[0]->alg;
        if ( host[which]->remote_path == (char*) NULL )
                host[which]->remote_path = host[0]->remote_path;

        /* always copy these */
        host[which]->remove = host[0]->remove;
        host[which]->oldtar = host[0]->oldtar;

        /* setup the overview status message */
        hdup_overview(NULL, NULL);
        hdup_overview("Program", PROGNAME);
        hdup_overview("Version", VERSION);
        
        if ( arg.scheme == DAILY || arg.scheme == WEEKLY || arg.scheme == MONTHLY) {

                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                /* okay all the options are parsed we can do some REAL work */
                hdup_setup(host, which);
                hdup_overview("Backup to",host[0]->path[0]);
                backup_local( host, which, arg);
        } else if ( arg.scheme == REMOTE ) {
                /* fix hostname argument for REMOTE */
                host[which]->name = arg.host;
                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                /* make the dirs */
                hdup_setup(host, which);
                hdup_overview("Backup to",host[0]->path[0]);
                /* remote inception, make the dir the remote host uses */
                hdup_mkdir(host[which]->remote_path, 0700);
                hdup_chown(host[which]->remote_path,host[which]->user);

                backup_remote(host, which, host[which]->date);
        } else if ( arg.scheme == RESTORE ) {
                char *arch;
                arch  = (char*)malloc(strlen(host[0]->path[0]) +
                                strlen(arg.host)+2);
                sprintf(arch,"%s/%s",host[0]->path[0],arg.host);
                if ( hdup_globfilecheck(arch,NULL) == 0 ) {
                        hdup_log("Cannot find archives for host");
                        hdup_cleanup(-1,NULL);
                }
                hdup_overview("Host", host[which]->name);
                hdup_overview("Date", host[which]->date);
                hdup_overview("Scheme", scheme[arg.scheme]);
                hdup_overview("Restore from",host[0]->path[0]);
                backup_restore(host, which, arg);
                hdup_cleanup(-1,NULL);
        }

        hdup_overview(NULL,NULL);
        if ( arg.scheme != REMOTE && host[0]->remote_host == NULL ) 
                hdup_overview("Archive size", (char*) (hdup_size (host[which]->archivename)));
        hdup_overview("Status", "succesfully performed backup");
        hdup_cleanup(0,host[which]);
        return 0;
}
