/* hduplib.c
 * lib with some general function
 * handy for hdup
 * 
 * Author: Miek Gieben
 * Date: Wed, 27 feb, 2002
 * 
 * Changelog
 *  27-02-2002: started
 *
 * Copyright:
 *
 *   This package is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; version 2 dated June, 1991.
 * 
 *   This package is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with this package; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *   02111-1307, USA.
 *
 */

#include"hdup.h"
#include"prototype.h"

char *dots   = "......................................";
char *spaces = "                                      ";
char *overview[OVERVIEW];

int hdup_getdate(char * d) {
        /* return the date in d
         * in the format:
         * DD-MM-YYYY
         * with 0-padding
         */
               
        time_t currenttime;
        struct tm *ltime;
        (void)time(&currenttime);
        ltime = localtime(&currenttime);

        snprintf(d,11, "%.2d-%.2d-%4d",ltime->tm_mday,
                          (ltime->tm_mon+1),
                          (ltime->tm_year+1900));
        return(0);
}

int hdup_setup ( phost_t host[], int which) {

        /* create /vol/backup/<hostname>
         *        /vol/backup/<hostname>/etc
         *        /vol/backup/<hostname>/<currentdate>
         */
        char *basename;
        char *d_etc;
        char *d_date;
        size_t baselen;

        /* 1 = \0 */
        baselen = strlen(host[0]->path[0])+ strlen(host[which]->name) +1;

        /* 2 = / \0 */
        basename = (char*) malloc ( baselen + 2);
        if (basename == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create the path of the directory */
        strncpy ( basename, host[0]->path[0], strlen(host[0]->path[0]) );
        strcat ( basename, "/");
        strncat ( basename, host[which]->name, strlen(host[which]->name) );
        host[which]->basename = basename;

        /* 2 = / \0 */
        d_date = (char *) malloc (baselen + 11 + 2);
        if (d_date == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create dir_date */
        strncpy ( d_date, basename, baselen);
        strcat  ( d_date, "/");
        strncat ( d_date, host[which]->date, strlen(host[which]->date));
        host[which]->dirname_date = d_date;

        /* 6 = / + 5 for '/etc\0' */
        d_etc = (char *) malloc ( baselen + 6);
        if (d_etc == NULL ) { hdup_log("Malloc error!!!"); hdup_cleanup(1,NULL);}
        /* create dir_etc */
        strncpy ( d_etc, basename, baselen);
        strcat  ( d_etc, "/etc");
        host[which]->dirname_etc = d_etc;

        (void)hdup_mkdir ( host[which]->basename , 0700);
        (void)hdup_chown ( host[which]->basename, host[which]->user);
        (void)hdup_mkdir ( host[which]->dirname_etc, 0700);
        (void)hdup_chown ( host[which]->dirname_etc, host[which]->user);
        (void)hdup_mkdir ( host[which]->dirname_date, 0700);
        (void)hdup_chown ( host[which]->dirname_date, host[which]->user);

        return 0;
}

int hdup_excludelist(phost_t host) {
        
        /* generate a file with the
         * patterns that should be 
         * excluded
         */

        FILE *fp;
        char *name;
        int i;

        name = (char*)malloc (strlen(host->dirname_etc) + strlen(EXCLUDELIST) + 8);
        strncpy(name, host->dirname_etc, strlen(host->dirname_etc)+1 );
        strcat(name, EXCLUDELIST);
        host->excludelist = name;
        
        fp = fopen(host->excludelist, "w");
        if ( fp != NULL ) {
                /* ok, went alright */
                for (i = 0; i < MAXDIR; i++) 
                        if ( host->exclude[i] != NULL ) {
                                fprintf(fp,"%s\n",host->exclude[i]);
                                /* also write to overview */
                                if ( i == 0 ) 
                                        hdup_overview("Excluding",host->exclude[i]);
                                else
                                        hdup_overview(host->exclude[i],NULL);
                        }
        fclose(fp); 
        }
        return 0;

}

int hdup_tarlist (phost_t host) {

        /* fill a file with the dirs to tar
         * this file is given as an option
         * to TAR later on, this file is
         * called "filelist"
         * This file is put in
         * /vol/backup/<hostname>/etc/
         */
        FILE *fp;
        char *name;
        int i;

        name = (char*)malloc (strlen(host->dirname_etc) + strlen(FILELIST) + 8);
        strncpy(name, host->dirname_etc, strlen(host->dirname_etc)+1 );
        strcat(name, FILELIST);
        host->filelist = name;
        
        fp = fopen(host->filelist, "w");
        if ( fp != NULL ) {
                /* ok, went alright */
                for (i = 0; i < MAXDIR; i++) 
                        if ( host->path[i] != NULL ) {
                                fprintf(fp,"%s\n",host->path[i]);
                                /* also write to overview */
                                if ( i == 0 ) 
                                        hdup_overview("Directories",host->path[i]);
                                else
                                        hdup_overview(host->path[i],NULL);
                        }
        fclose(fp); 
        }
        return 0;
}

int hdup_archive (phost_t host, int s) {
        /* fill hdup with the archive name */
              
        size_t len;
        len = 7 + strlen(host->dirname_date) +
                strlen(host->name) + strlen(host->date) + strlen(scheme[s]);
        len = len + 5; /* extra space */

        /* /vol/backup/<host>/<date>/<host>.<date>.<scheme>.tar.bz2 */
        host->archivename = (char*) malloc(len);
        if ( strcmp (host->tar_opt, TAR_BZIPO) == 0 ) {
                hdup_overview("Compression","(old tar) bzip2");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],BZIP_EXT);
                return 0;
        }
        if ( strcmp (host->tar_opt, TAR_GZIP) == 0 ) {
                hdup_overview("Compression","gzip");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],GZIP_EXT);
                return 0;
        }
        if ( strcmp (host->tar_opt, TAR_NONE) == 0 ) {
                hdup_overview("Compression","none");
                sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                        host->name,host->date,scheme[s],NONE_EXT);
                return 0;
        }
        /* default */
        hdup_overview("Compression","bzip2");
        sprintf(host->archivename, "%s/%s.%s.%s%s",host->dirname_date,
                host->name,host->date,scheme[s],BZIP_EXT);
        return 0;
}

int hdup_inclist (phost_t host, int s) {
        size_t len;

        char *inc_day, *inc_week, *inc_month = NULL;
        /* okay new idea */
        /*
         * month        -> delete month/week/day -> use month
         * weekly       -> if no weekly -> cp month to weekly
         *              | then use weekly + rm daily
         * daily        -> if no daily -> cp weekly to daily
         *              | then use daily
         */

        if ( s == REMOTE ) { 
                hdup_log("internal error: wrong function call");
                hdup_cleanup(1, host);
        }

        /* len (/inclist.) = 9 */
        len = strlen(host->dirname_etc) + 9 + 8;

        inc_month = (char*) malloc(len);
        if ( inc_month == NULL ) 
                hdup_log("internal error");
        snprintf(inc_month, len,"%s/inclist.%s",host->dirname_etc,scheme[MONTHLY]);

        inc_week = (char*) malloc(len);
        if ( inc_week == NULL ) 
                hdup_log("internal error");
        snprintf(inc_week, len, "%s/inclist.%s",host->dirname_etc,scheme[WEEKLY]);

        inc_day = (char*) malloc(len);
        if ( inc_day == NULL ) 
                hdup_log("internal error");
        snprintf(inc_day, len, "%s/inclist.%s",host->dirname_etc,scheme[DAILY]);

        switch (s) {
                case MONTHLY:
                        host->inclist = inc_month;
                        /* remove it */
                        (void*) unlink(host->inclist);
                        (void*) unlink(inc_week);
                        (void*) unlink(inc_day);
                        free (inc_week); free(inc_day);
                        break;
                case WEEKLY:
                        host->inclist = inc_week;
                        if ( hdup_checkfile(inc_month) == 1 ) {
                                /* does exist cp monthly to weekly */
                                (void)hdup_cp(inc_month,inc_week);
                                (void*)unlink(inc_day);
                        } else {
                                /* not usable */
                                hdup_overview("Status","could not found previous backup list");
                                rmdir(host->dirname_date);
                                hdup_cleanup(-2,host);
                        }
                        free(inc_month); free(inc_day);
                        break;
                case DAILY:
                        host->inclist = inc_day;
                        if ( hdup_checkfile(inc_week) == 1 ) {
                                /* does exist * cp weekly to daily */
                                (void)hdup_cp(inc_week,inc_day );
                        } else {
                                /* not usable */
                                hdup_overview("Status","could not found previous backup list");
                                rmdir(host->dirname_date);
                                hdup_cleanup(-2,host);
                        }
                        free(inc_month); free(inc_week);
                        break;
        }
        return 0;
}

int hdup_mv (char *from,char *to ) {
        int pid, status;

        char *command;
        command = (char*)malloc(strlen(from) + strlen(to) + 24 );

        sprintf(command, "/bin/mv -f %s %s 2>/dev/null", from, to);
                        
        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL); /* MIEK hmm */
        }
        if ( pid == 0 ) {
                /* child */
                /* compose move */
               execl("/bin/sh", "sh", "-c",
                               command,
                               NULL);
               hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                return 1;
        }        
        return 0;
}

int hdup_cp (char *from, char *to ) {
        int pid, status;

        char *command;
        command = (char*)malloc(strlen(from) + strlen(to) + 27 );

        sprintf(command, "/bin/cp -f %s %s 2>/dev/null", from, to);
                        
        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
               execl("/bin/sh", "sh", "-c",
                               command,
                               NULL);
               hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("could not perform the copy");
                return 1;
        }        
        return 0;
}

int hdup_lock(phost_t host, int l) {
        /* set a LOCK FILE 
         * in host->basename/LOCK
         */
        size_t len;
        char *lockfile;
        FILE *lock;

        /* /LOCK  = 5  + extra */
        len = strlen(host->basename) + 7;

        lockfile = (char *)malloc(len);
        strcpy( lockfile, host->basename);
        strcat( lockfile, "/LOCK");

        switch (l) {
                case LOCK:
                        if (hdup_checkfile(lockfile) == 1 ||
                            hdup_checkfile(lockfile) == 2 ) {
                                /* lockfile exists */
                                hdup_overview("Status", "LOCK found");
                                hdup_log("LOCK found");
                                hdup_cleanup(-1, NULL);
                        }
                        lock = fopen(lockfile, "w");
                        if ( lock != NULL) {
                                /* ok */
                                fclose(lock);
                        } else {
                                /*  hmm */
                                hdup_overview("Status","could not create LOCK");
                                hdup_log("Could not create LOCK");
                                hdup_cleanup(-1, NULL);
                                return 1;
                        }
                        break;
                case UNLOCK:
                        if (hdup_checkfile(lockfile) == 0 ) {
                                /* hmmm, nothing */
                                return 0;
                        } else {
                                /* lock found */
                                unlink(lockfile);
                        }
                        break;
        }
        return 0;
}

int hdup_scp (phost_t host) {
        /* scp the archive to remote host:/tmp/backup */

        /* need option_t for identity */
        int pid, status;
        char *scp_arg;

        scp_arg = (char*)malloc (strlen(host->scp_user) + 
                        strlen(host->remote_host ) + 14);
        sprintf(scp_arg, "%s@%s:%s/", host->scp_user, host->remote_host,host->remote_path);

        if ( hdup_scp_mkdir ( host) != 0 ) {
                hdup_log("Could not perform the remote copy");
                return 1;
        }

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
                /* compose scp */
                if ( host->identity == NULL ) {
                        execl(SCP,"scp",
                                        host->archivename,
                                        scp_arg,
                                        NULL);
                        hdup_cleanup(1,NULL);
                } else {
                        execl(SCP,"scp","-i",
                                        host->identity,
                                        host->archivename,
                                        scp_arg,
                                        NULL);
                        hdup_cleanup(1,NULL);
                }

        }

        free (scp_arg);

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("Could not perform the remote copy");
                return 1;
        }        
        return 0;
}

int hdup_scp_mkdir(phost_t host){ 
        
        int pid, status;
        char *ssh_arg;

        ssh_arg = (char*)malloc (strlen(host->scp_user) + 
                        strlen(host->remote_host ) + 3);
        sprintf(ssh_arg, "%s@%s", host->scp_user, host->remote_host);

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
                /* compose scp */
                if ( host->identity == NULL ) {
                        execl("/usr/bin/ssh","shh",ssh_arg,
                                        "mkdir","-,", "1777","-p",host->remote_path,
                                        NULL);
                        hdup_cleanup(1,NULL);
                } else {
                        execl("/usr/bin/ssh","ssh","-i",
                                        host->identity,
                                        ssh_arg,
                                        "mkdir","-m", "1777","-p",host->remote_path,
                                        NULL);
                        hdup_cleanup(1,NULL);
                }

        }

        free (ssh_arg);

        while (wait(&status) != pid)
                /* empty */ ;

        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("Could not perform the remote mkdir");
                return 1;
        }        
        return 0;
}

int hdup_crypt (phost_t host) {
        /* crypt the archive */

        /* mcrypt -a opt.alg -f opt.keypath host->archivename */

        int pid, status;

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
                /* compose mcrypt, -u to delete archive */
                execl(MCRYPT,"mcrypt","-u",
                                "-a", host->alg,
                                "-f", host->keypath, 
                                host->archivename,
                                NULL);
                hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;
        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("could not perform the encryption");
                return 1;
        } else {
                /* if succesfull add the .nc extension */
                /* WHY? [miek] doesn't mcrypt do this */
                /* should do this for admin sake, hdup must
                 * know that this is added by mcrypt
                 */
                strncat(host->archivename, MCRYPT_EXT, 3);
                return 0;
        }
}

int hdup_tar (phost_t host) {
        int pid, status;

        /* tar cvfjg /tmp/back2.tar.bz2 inclist.SCHEME -T file-from */

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,host);
        }
#ifdef DEBUG
        printf("%s\n",host->tar_opt);
        printf("%s\n",host->filelist);
        printf("%s\n",host->inclist);
#endif
                       
        if ( pid == 0 ) {
                /* child */
                /* compose tar */
                execl(TAR,"tar",
                                host->tar_opt,
                                host->archivename,
                                "--files-from",
                                host->filelist,
                                "--listed-incremental",
                                host->inclist,
                                "--exclude-from",
                                host->excludelist,
                                "--ignore-failed-read",
                                NULL);
                hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;
        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("error while tarring");
                return 1;
        }
        return 0;       /* tar was succesfull */
}

int hdup_untar (char *archive, char *extractdir, int compression) {
        int pid, status;
        char *tar_opt;

        tar_opt = (char*)malloc(8);
        switch (compression) {
                case NONE:
                        tar_opt = "xpfC";
                        break;
                case GZIP:
                        tar_opt = "xpfCz";
                        break;
                case BZIP:
                        tar_opt = "xpfCj";
                        break;
                case BZIPO:
                        tar_opt = "xpfCI";
                        break;
                default:
                        tar_opt = "xpfCj";
                        break;
        }        
        
        /*
         * tar xvvvpfCz
         * /vol/backup/elektron/15-03-2002/elektron.15-03-2002.daily.tar.gz
         * /tmp/BLA --incremental
         */

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
                       
        if ( pid == 0 ) {
                /* child */
                /* compose tar */
                execl(TAR,"tar",
                                tar_opt,
                                archive,
                                extractdir,
                                "--incremental",
                                "--ignore-failed-read",
                                NULL);
                hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;
        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("error while untarring");
                return 1;
        }
        return 0;       /* untar was succesfull */
}

int hdup_decrypt (char *archivename, phost_t host) {
        /* decrypt the archive */

        /* mdecrypt -a opt.alg -f opt.keypath archivename */

        int pid, status;

        if ((pid = fork()) < 0) {
                perror("fork");
                hdup_cleanup(1,NULL);
        }
        if ( pid == 0 ) {
                /* child */
                /* compose mdecrypt, */
                execl(MDECRYPT,"mdecrypt",
                                "-a", host->alg,
                                "-f", host->keypath, 
                                archivename,
                                NULL);
                hdup_cleanup(1,NULL);
        }

        while (wait(&status) != pid)
                /* empty */ ;
        if ( status != 0 ) {
                /* shit hit the fan */
                hdup_log("could not perform the decryption");
                return 1;
        } else {
                return 0;
        }
}

int hdup_system (char *program, char *arg, ...) {
        /* do a fork for the program and its arguments
         * error-output is diverted to /dev/null
         * This is not finished!
         */
        /*
        va_list ap;
        char *s;
        char *command;
        size_t l,i;

        l = strlen(program) + 2;
        command = (char*)malloc( l );

        va_start(ap, arg);
        while (*arg) {
                s = va_arg(ap, char *);
                i = strlen(s);
                l = l + i + 1;  adept the size for command array 
                command = (char*)realloc( command, l );  new size 
                memcpy(command + l, s, i-1);
                *arg++;
        }
        va_end(ap);
        printf("%d\n",l);
        printf("%s\n",command);
         * NOT YET FINISHED
         */
        return 0;
}

void hdup_log (char * message) {
        /* log with newline */
        if ( quiet == 0 )
                fprintf(stderr, "%s\n", message);
}

void hdup_log_nn (char * message) {
        /* log without newline */
        if ( quiet == 0 )
                fprintf(stderr, "%s", message);
}

void hdup_cleanup (int exitcode, phost_t host) {
        /* exit and do some cleanups */
        
        
        if ( host != NULL ) 
                hdup_lock(host, UNLOCK);

        /* newline */
        hdup_overview(NULL, NULL);

        /* no explicit status error */
        if ( exitcode > 0 ) 
                        hdup_overview("Status", "An error occured");
        
        /* do not print overview */
        if ( exitcode == -1 )
                        exit(1);

        /* explicit status error 
         * already put in overview
         */
        if ( exitcode == -2 )
                        exitcode = 2;

        hdup_print();
        exit (exitcode);
}

int hdup_overview (char *info, char *value) {
        
        /* fill the overview */
        int li, lv, i;
        static int row_count;

        if (row_count > OVERVIEW ) {
                hdup_log("overview overflow");
                hdup_cleanup(1, NULL);
        }

        if ( info == NULL && value == NULL ) {
                /* insert newline */
                overview[row_count] = (char*) malloc(2);
                *overview[row_count] = '\0';
                strncpy(overview[row_count], "\n\0",2);
                row_count++;
                return 0;
        }

        if ( value == NULL ) {
                /* just a message */
                li = strlen(info);
                i = DOTEND + 3;
                overview[row_count] = (char*) malloc(li+i+10);
                *overview[row_count] = '\0';

                strncpy(overview[row_count], spaces, DOTEND+3);
                strncat(overview[row_count], info, li);
                strncat(overview[row_count], "\n\0", 2);
        } else if ( info == NULL ) {
                /* illegal [MIEK] */
        } else {
                /* a info.....: value message */
                li = strlen(info); lv = strlen(value);
                i = DOTEND - li;
                overview[row_count] = (char*) malloc(li+lv+i+10);
                *overview[row_count] = '\0';

                strncpy(overview[row_count], info, li+1);
                strncat(overview[row_count], dots, i);
                strncat(overview[row_count], ":  ", 3);
                strncat(overview[row_count], value, lv+1);
                strncat(overview[row_count], "\n", 1);

        }
        row_count++;
        return 0;
}

void hdup_print (void) {
        int i = 0;

        while ( overview[i] != (char*)NULL ) {
                printf("%s", overview[i]);
                i++;
        }
}

char * hdup_size (char *filepath) {

        /* give back size in file,
         * 5 k, 5M, 5G 
         */
        off_t size; /* size in bytes */
        float s = 0;
        char suffix = '\0';
        char *filesize = (char*)malloc(6);

        struct stat *buf;
        buf = (struct stat*) malloc(sizeof(struct stat));

        if ( stat(filepath, buf) == 0 ) {
                size = buf->st_size;

                if ( size > GIGABYTE ) {
                        s = (float) size / (float) GIGABYTE;
                        suffix = 'G';
                        sprintf(filesize,"%.1f%c",s, suffix);
                        return filesize;
                }
                else if (size > MEGABYTE) {
                        s = (float) size / (float) MEGABYTE;
                        suffix = 'M';
                        sprintf(filesize,"%.1f%c",s, suffix);
                        return filesize;
                }
                else if (size > KILOBYTE) {
                       s = (float) size / (float) KILOBYTE;
                       suffix = 'k';
                       sprintf(filesize,"%.0f%c",s, suffix);
                       return filesize;
                }
                sprintf(filesize,"%d%c",(int)size, suffix);
                free(buf);
                return filesize;
        }
        return (char*)NULL;
}

char * hdup_lastname (char * filepath) {
        char *w, *last, *t;

        t = strrchr (filepath, '/');
        if ( t != NULL ) {
                /* advance 1 further */
                t++;
                last = (char*) malloc (strlen(filepath));
                w = last;
                while ( *t != '\0' ) {
                        *last = *t;
                        last++; t++;
                }
                *last = '\0';
                last = w; /* reset */
                return last;
        }
        return (char*)NULL;
}

int hdup_checkfile (char *filepath) {
        
        /*
         * returns
         * 1: file
         * doesn't do globbing
         */
         
        struct stat *buf;
        buf = (struct stat*) malloc(sizeof(struct stat));

        if ( stat(filepath, buf) == 0 ) {
                return ( S_ISREG(buf->st_mode) );
        }
        return 0;
}

int hdup_checkdir ( char *dirpath) {

        struct stat *buf;
        buf = (struct stat*) malloc(sizeof(struct stat));

        if ( stat(dirpath, buf) == 0 ) {
                return ( S_ISDIR(buf->st_mode) );
        }
        return 0;

}

int hdup_globfilecheck( char *globpath, char *firstmatch) {
        /* use glob(3) to search for files */

        /* returns the number of found matches */

        int i;
        glob_t globbuf;
        
        globbuf.gl_offs = 0;
        glob(globpath,GLOB_DOOFFS,NULL, &globbuf);
        i = globbuf.gl_pathc;

        if ( firstmatch != NULL && i != 0 ) {
                strcpy(firstmatch, globbuf.gl_pathv[0]);
        }
        globfree(&globbuf);
        return i;
}

int hdup_checkperm (char *filepath) {
        return 0;
}

int hdup_mkdir(char *dirpath, mode_t mode) {

        char *t, *d;
        size_t l = strlen(dirpath);

        d = (char *)malloc(l+1);
        strncpy(d,dirpath, l);

        if ( *(d+l) != '/' ) 
                *(d+l) = '/';

        for ( t = d ; *t != '\0'; t++ ) {
                if ( *t == '/' ) {
                        *t = '\0';
                        mkdir(d, mode);
                        *t = '/';
                }
        }
        return 0;
}

int hdup_chown (char *path, char *user) {

        struct passwd *entry;
        uid_t uid; gid_t gid;

        if ( ( entry = getpwnam(user)) != NULL ) {
                gid = entry->pw_gid;
                uid = entry->pw_uid;
                return ( chown(path, uid, gid));
        } else {
                return -1;
        }
}

int hdup_compression (char *archivename) {
        /* return the encryption use
         * and an archive
         */
        if ( archivename == NULL )
                return -1;

        /* check for BZIP */
        if (strstr(archivename, BZIP_EXT) != NULL )
                return BZIP;
        if (strstr(archivename, GZIP_EXT) != NULL )
                return GZIP;
        /* should be none than */
        return NONE;
}


int usage() {
        fprintf(stderr,"\nUsage:\n");
        fprintf(stderr,"\thdup -c config [ OPTION ] SCHEME HOST [ @REMOTE_HOST ] (1st format)\n");
        fprintf(stderr,"\thdup -c config [ OPTION ] restore HOST DATE DIRECTORY  (2nd format)\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"Arguments:\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\tSCHEME");
        fprintf(stderr,"\t\tdaily, weekly, monthly, remote or restore\n");
        fprintf(stderr,"\tHOST");
        fprintf(stderr,"\t\tspecify which host to backup\n");
        fprintf(stderr,"\t\t\tor which host to restore\n");
        fprintf(stderr,"\t@REMOTE_HOST");
        fprintf(stderr,"\twhere to transfer the archive to\n");
        fprintf(stderr,"\t\t\tonly used when SCHEME is daily, weekly or monthly\n");
        fprintf(stderr,"\tDATE");
        fprintf(stderr,"\t\twhen restoring: restore up to this date\n");
        fprintf(stderr,"\tDIRECTORY");
        fprintf(stderr,"\twhen restoring: restore to this directory\n");

        fprintf(stderr,"\nOptions:\n\n");
        fprintf(stderr,"\t-c config\tlocation of the configuration file\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-C compression\tspecify either 'bzip', 'gzip' or 'none' for the archive\n");
        fprintf(stderr,"\t\t\tcompression. Defaults to bzip\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-u user\t\tspecify user to chown archives to\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-s user\t\tscp the archives under this user\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-i identity\tspecify ssh identity file\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-a algorithm\tspecify algorithm to use when encrypting the archive\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-k keyfile\tspecify the file with the encyption key\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-o\t\told tar. Older versions of tar use -I for bzip compression, use this\n");
        fprintf(stderr,"\t\t\tif your tar doesn't understand de -j option. Only valid with -C\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-r\t\tremove the archive after transferring it to remote_host\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-q\t\tno logging at all, also no overview message\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"\t-h\t\tthis help\n");
        fprintf(stderr,"\n");
        return 0;
}
